/*
 * LAMIP - global definitions.
 *
 * don't include this in your plugins
 * this will be include automatically, just here for sharing purpose
 * but don't forget that you could use theses functions into your plugin
 *
 * instead use for :
 *	input plugin   -> plug_in.h
 *	output plugin  -> plug_out.h
 *	control plugin -> plug_control.h
 *
*/

#ifndef __LAMIP_H_
#define __LAMIP_H_

#include <stdlib.h> /* for size_t */
#include <stdarg.h> /* lamip_send_message */

#ifndef LAMIP_VERSION
#define LAMIP_VERSION "0.0.3+20050306"
#endif

typedef struct _lamipPluginHandle	lamipPluginHandle;
typedef struct _lamipPlaylist		lamipPlaylist;
typedef struct _lamipSonginfo		lamipSonginfo;

typedef enum {
  PCM_FORMAT_S8 = 0,
  PCM_FORMAT_U8,
  PCM_FORMAT_S16_LE,
  PCM_FORMAT_S16_BE,
  PCM_FORMAT_U16_LE,
  PCM_FORMAT_U16_BE,
  /* haven't supported */
  PCM_FORMAT_S24_LE,	/* low three bytes */
  PCM_FORMAT_S24_BE,	/* low three bytes */
  PCM_FORMAT_U24_LE,	/* low three bytes */
  PCM_FORMAT_U24_BE,	/* low three bytes */
  PCM_FORMAT_S32_LE,
  PCM_FORMAT_S32_BE,
  PCM_FORMAT_U32_LE,
  PCM_FORMAT_U32_BE

} lamipPCMFormat;


/*
 * Structure contained in all plugins
*/
struct		_lamipPluginBase {
  const char*	name;
  const char*	description;
  int		(*init)(const lamipPluginHandle*);
  void		(*cleanup)(void);
};

/* infos about pcm datas */
typedef struct {
  lamipPCMFormat	format;
  unsigned long		samplerate;
  unsigned long		channels;
} lamipPCMInfo;

#ifdef __cplusplus
extern "C" {
#endif

  /*
   * set the track lamip_songinfo_{get,set}* should work on
   * valid range: [0, total_tracks - 1]
   */
int		lamip_songinfo_getTotalTracks(const lamipSonginfo*);
int		lamip_songinfo_setActiveTrack(const lamipSonginfo*, int);


  /*
   * configuration functions. Use it to read/save informations
   * for your plugins (in the configuration plugin specific function)
   * all returned values should be free
   *
   * we could maybe add some conversion functions
   * like str2int, str2float, str2double, etc...
   * depend of users needing
   */

/* free the returned value */
char*	lamip_cfg_get(const lamipPluginHandle*, const char*);
int	lamip_cfg_getInt(const lamipPluginHandle*, const char*);
int	lamip_cfg_getBool(const lamipPluginHandle*, const char*);
/* see if key exists */
int	lamip_cfg_getExist(const lamipPluginHandle*, const char*);
void	lamip_cfg_set(lamipPluginHandle*, const char*, const char*);
void	lamip_cfg_setInt(lamipPluginHandle*, const char*, int);
void	lamip_cfg_setBool(lamipPluginHandle*, const char*, int);
void	lamip_cfg_unset(lamipPluginHandle*, const char*);
/* retreive configuration keys in a array */
char**	lamip_cfg_getKeys(const lamipPluginHandle*);


  /*
   * write a message to lamip, (usefull to sow into a control)
   */
void	lamip_send_message(char*, ...);


#ifdef __cplusplus
}
#endif

#endif /* __LAMIP_H_ */
