#ifndef __PLUG__CONTROL_H
#define __PLUG__CONTROL_H

#include "lamip.h"


/*
 * This structure is used in the 'event' function of the control plugin.
 * each event could send some datas to specifie some things about the event
 *
 * for example the EVENT_SONG_CHANGED send the new position of the song
 *
*/

struct _lamipEvent {
  enum {
    EVENT_SONG_CHANGED,
    EVENT_SONG_BEGIN,
    EVENT_SONG_END,
    EVENT_POS_CHANGED,
    EVENT_PAUSED,
    EVENT_UNPAUSED,
    EVENT_META_CHANGED,
    EVENT_MESSAGE,
    EVENT_EXIT,
    EVENT_VOLUME_CHANGED
  } type;
  union {
    struct {				/* data for EVENT_SONG_CHANGED */
      lamipPlaylist*	pl;
      int		pos;
      int		trk;

    }			song_changed;
    unsigned long	pos_changed;	/* data for EVENT_POS/VOLUME_CHANGED */
    struct {
      char*		key;
      char*		val;

    }			meta_changed;	/* data for EVENT_META_CHANGE */
    char*		message;	/* data for EVEN_MESSAGE */
  } data;
};


/*
 * Your plugin need to define theses functions.
 * it will return this struture at the loading time
 * the 'common' part is described in lamip.h
*/
typedef struct {
  struct _lamipPluginBase	common;
  void				(*run)(void);
  void				(*signal)(const struct _lamipEvent*);
} ControlPlugin;

typedef enum {
  PLUGIN_TYPE_INPUT,
  PLUGIN_TYPE_EFFECT,
  PLUGIN_TYPE_OUTPUT,
  PLUGIN_TYPE_CONTROL
}	lamipPluginType;

enum _lamipOrder {
  ORDER_ONE_BY_ONE,
  ORDER_REPEAT_PLAYLIST,
  ORDER_REPEAT_FILE,
  ORDER_RANDOM
};

#ifdef __cplusplus
extern "C" {
#endif

  /*
   * General access
  */
int		lamip_play(void);
void		lamip_stop(void);
void		lamip_pause(int);
void		lamip_prev(void);
void		lamip_next(void);
void		lamip_seek(unsigned long);


  /*
   * Act on the playing_thread
  */
void		lamip_setOrder(enum _lamipOrder);/* set playing order */
/* make the selected song next song (ignore play order) */
void		lamip_setFlagFollow(int);
void		lamip_setFlagStop(int); /* stop after current song */
void		lamip_setChannels(unsigned long);
void		lamip_setReplayGain(int);
void		lamip_setGapless(int);
int		lamip_getGapless(void);

const char*	lamip_getAboutText(void);

unsigned long	lamip_getSongLen(void);
#if 0
void		lamip_leaving(void); /* inform core that we're leaving */
#else
void		lamip_setVolume(unsigned int);
unsigned long	lamip_getCurrentSongPos(void);
int		lamip_getPaused(void);
#endif


  /*
   * lamipPlugin Managment
   * you could get some information to show on your UI
   */
lamipPluginHandle*	lamip_plugin_getHandle(lamipPluginType);
lamipPluginHandle*	lamip_plugin_getNext(lamipPluginHandle*);
lamipPluginHandle*	lamip_plugin_search(const char* /*name*/);
lamipPluginHandle*	lamip_plugin_searchInType(const char*, lamipPluginType);
char*			lamip_plugin_getName(lamipPluginHandle*);
char*			lamip_plugin_getDescription(lamipPluginHandle*);
int			lamip_plugin_activate(lamipPluginHandle*);
void			lamip_plugin_deactivate(lamipPluginHandle*);


  /*
   * lamipPlaylists Management
   */
lamipPlaylist*	lamip_playlist_new(void);
void		lamip_playlist_destroy(lamipPlaylist*);
int		lamip_playlist_set(lamipPlaylist*, int, int);
void		lamip_playlist_get(lamipPlaylist**, int*, int*);
int		lamip_playlist_getLength(lamipPlaylist *);
lamipSonginfo*	lamip_playlist_getSong(lamipPlaylist *, int);
/* 
 * return the new position of pos after sorting
 * shuffle if cmp == NULL
 */
int		lamip_playlist_sort(lamipPlaylist* pl, int pos,
    int (*)(const lamipSonginfo *, const lamipSonginfo *));
int		lamip_playlist_reverse(lamipPlaylist*, int);
int		lamip_playlist_append(lamipPlaylist*, lamipSonginfo*);
int		lamip_playlist_prepend(lamipPlaylist*, lamipSonginfo*);
int		lamip_playlist_remove(lamipPlaylist*, int);
int		lamip_playlist_clear(lamipPlaylist*);
void		lamip_playlist_foreach(lamipPlaylist*, void (*)(lamipSonginfo*, void*), void*);


  /*
   * Songinfo Management functions to retreive datas from files info
   * such as ID3, APE tags
   */
lamipSonginfo*	lamip_songinfo_new(void);
void		lamip_songinfo_destroy(lamipSonginfo*);
int		lamip_songinfo_read(const char*, lamipSonginfo*);
unsigned long	lamip_songinfo_getLength(const lamipSonginfo*);
void		lamip_songinfo_setLength(lamipSonginfo*, unsigned long);
/* you must free returned value */
char*		lamip_songinfo_getFilename(const lamipSonginfo*);
char*		lamip_songinfo_getTitle(const lamipSonginfo*);
char*		lamip_songinfo_getArtist(const lamipSonginfo*);
char*		lamip_songinfo_getAlbum(const lamipSonginfo*);
char*		lamip_songinfo_getTrack(const lamipSonginfo*);
char*		lamip_songinfo_getType(const lamipSonginfo*);
char*		lamip_songinfo_getYear(const lamipSonginfo*);
char*		lamip_songinfo_getField(const lamipSonginfo*, const char*);


  /*
   * lamip global configuration access
   *
   */
char*		lamip_cfg_coreGet(const char*);
void		lamip_cfg_coreSet(const char* /*key*/, const char* /*value*/);
char**		lamip_cfg_getCoreKeys(void);

#ifdef __cplusplus
}
#endif

#endif /*  __PLUG__CONTROL_H */
