/*
 * LAMIP - input plugin definitions
 *
*/

#ifndef __PLUG__IN_H_
#define __PLUG__IN_H_

#include "lamip.h"

typedef struct _lamipURL lamipURL;

typedef struct	_InputPlugin {
  struct _lamipPluginBase	common;
  void				(*decode)(lamipURL*, int /*sub_trk*/);
  void				(*set_song_info)(lamipURL*, lamipSonginfo*);
}		InputPlugin;


#ifdef __cplusplus
extern "C" {
#endif

/* should be really call in init function */
void	lamip_set_mime_type(lamipPluginHandle*, const char* /*extention*/, const char* /*mime-type*/);


int		lamip_isContinue(void);
int		lamip_isSeek(void);
unsigned long	lamip_isSeekGetAndReset(void);

/*
 * before writting data.. open audio device
 * then write data and...after .. close it
 * it's like writing in a file !
 *
 * open :(1.pcm info, 2.songlen)
 * 3 - song length in millisecond
*/
int	lamip_open(lamipPCMInfo*, unsigned int);
void	lamip_close(void);
int	lamip_writeData(void* /*data*/, size_t /*length*/);

/*
 * ask core to drain/drop the output buffer
 * useful when seek or lamip_doAbort() == 1
 * (do nothing for moment)
*/
void	lamip_drop(void);
void	lamip_drain(void);

/*
 * some functions to use, so you could use it as a
 * file via http/ftp/ssh/rtp etc... reader.
 * depend of lamip configuration time option
 *
 * currently only 'file://', 'http://' and 'ftp://' are supported
 */

//lamipURL*	lamip_url_open(const char* /*url/path*/);
//int		lamip_url_close(lamipURL*);
size_t		lamip_url_read(lamipURL*, void* /*buf*/, size_t);
int		lamip_url_seekable(lamipURL*);
int		lamip_url_seek(lamipURL*, long /*offset*/);
int		lamip_url_eos(lamipURL*);
char*		lamip_url_getURL(lamipURL*);

/*
 * in the (*set_song_info) use these function to set
 * value for lamipSonginfo parameter
 */
int	lamip_songinfo_setTotalTracks(lamipSonginfo*, int);
void	lamip_songinfo_setFilename(lamipSonginfo*, const char*);
void	lamip_songinfo_setTitle(lamipSonginfo*, const char*);
void	lamip_songinfo_setArtist(lamipSonginfo*, const char*);
void	lamip_songinfo_setAlbum(lamipSonginfo*, const char*);
void	lamip_songinfo_setTrack(lamipSonginfo*, const char*);
void	lamip_songinfo_setType(lamipSonginfo*, const char*);
void	lamip_songinfo_setYear(lamipSonginfo*, const char*);

/* set others specific value that some UI could manage */
void	lamip_songinfo_setField(lamipSonginfo*, const char* /*field*/ , const char* /*value*/);


  // special case for the playlist plugin
InputPlugin *lamip_plugin_getInputPlugin(const char* filename, const char* mime_type);

#ifdef __cplusplus
}
#endif

#endif /* __PLUG__IN_H_ */
